@@ -11,6 +11,10 @@ class Api(object):
     def api_key(self):
         return self.api.api_key
 
+    @property
+    def api_url(self):
+        return self.api.api_url
+
     @property
     def app_url(self):
         return self.api.app_url
@@ -31,9 +35,22 @@ class Api(object):
     def viewer(self):
         return self.api.viewer()
 
+    def list_projects(self, entity=None):
+        return self.api.list_projects(entity=entity)
+
+    def format_project(self, project):
+        return self.api.format_project(project)
+
+    def upsert_project(self, project, id=None, description=None, entity=None):
+        return self.api.upsert_project(project, id=id, description=description,
+                                       entity=entity)
+
     def settings(self, *args, **kwargs):
         return self.api.settings(*args, **kwargs)
 
+    def clear_setting(self, *args, **kwargs):
+        return self.api.clear_setting(*args, **kwargs)
+
     def set_setting(self, *args, **kwargs):
         return self.api.set_setting(*args, **kwargs)
 
@@ -43,11 +60,14 @@ class Api(object):
     def download_urls(self, *args, **kwargs):
         return self.api.download_urls(*args, **kwargs)
 
+    def create_anonymous_api_key(self):
+        return self.api.create_anonymous_api_key()
+
     def sweep(self, *args, **kwargs):
         return self.api.sweep(*args, **kwargs)
 
     def upsert_sweep(self, *args, **kwargs):
-        return self.api_upsert_sweep(*args, **kwargs)
+        return self.api.upsert_sweep(*args, **kwargs)
 
     def register_agent(self, *args, **kwargs):
         return self.api.register_agent(*args, **kwargs)
