@@ -25,6 +25,26 @@ from wandb.util import (
 logger = logging.getLogger("wandb")
 
 
+def file_policy_to_enum(policy):
+    if policy == "now":
+        enum = wandb_internal_pb2.FilesItem.PolicyType.NOW
+    elif policy == "end":
+        enum = wandb_internal_pb2.FilesItem.PolicyType.END
+    elif policy == "live":
+        enum = wandb_internal_pb2.FilesItem.PolicyType.LIVE
+    return enum
+
+
+def file_enum_to_policy(enum):
+    if enum == wandb_internal_pb2.FilesItem.PolicyType.NOW:
+        policy = "now"
+    elif enum == wandb_internal_pb2.FilesItem.PolicyType.END:
+        policy = "end"
+    elif enum == wandb_internal_pb2.FilesItem.PolicyType.LIVE:
+        policy = "live"
+    return policy
+
+
 class BackendSender(object):
     class ExceptionTimeout(Exception):
         pass
@@ -65,20 +85,24 @@ class BackendSender(object):
         rec.output.CopyFrom(o)
         self._queue_process(rec)
 
+    def _send_history(self, history):
+        rec = self._make_record(history=history)
+        self._queue_process(rec)
+
     def send_history(self, data, step):
-        rec = wandb_internal_pb2.Record()
         data = data_types.history_dict_to_json(self._run, data, step)
-        history = rec.history
+        history = wandb_internal_pb2.HistoryData()
         for k, v in six.iteritems(data):
             item = history.item.add()
             item.key = k
             item.value_json = json_dumps_safer_history(v)
-        self._queue_process(rec)
+        self._send_history(history)
 
     def _make_run(self, run):
         proto_run = wandb_internal_pb2.RunData()
         run._make_proto_run(proto_run)
         proto_run.start_time.GetCurrentTime()
+        proto_run.host = run._settings.host
         if run._config is not None:
             config_dict = run._config._as_dict()
             self._make_config(config_dict, obj=proto_run.config)
@@ -191,9 +215,10 @@ class BackendSender(object):
 
     def _make_files(self, files_dict):
         files = wandb_internal_pb2.FilesData()
-        for path in files_dict["files"]:
+        for path, policy in files_dict["files"]:
             f = files.files.add()
-            f.path[:] = path
+            f.path = path
+            f.policy = file_policy_to_enum(policy)
         return files
 
     def _make_record(
@@ -202,6 +227,7 @@ class BackendSender(object):
         config=None,
         files=None,
         summary=None,
+        history=None,
         stats=None,
         exit=None,
         artifact=None,
@@ -213,6 +239,8 @@ class BackendSender(object):
             rec.config.CopyFrom(config)
         if summary:
             rec.summary.CopyFrom(summary)
+        if history:
+            rec.history.CopyFrom(history)
         if files:
             rec.files.CopyFrom(files)
         if stats:
@@ -253,8 +281,8 @@ class BackendSender(object):
         # returns response, err
         return rsp
 
-    def send_run(self, run_dict):
-        run = self._make_run(run_dict)
+    def send_run(self, run_obj):
+        run = self._make_run(run_obj)
         rec = self._make_record(run=run)
         self._queue_process(rec)
 
@@ -268,13 +296,25 @@ class BackendSender(object):
         rec = self._make_record(summary=summary)
         self._queue_process(rec)
 
-    def send_run_sync(self, run_dict, timeout=None):
-        run = self._make_run(run_dict)
-        req = self._make_record(run=run)
+    def _send_run_sync(self, run, timeout=None):
+        """Send synchronous run object waiting for a response.
 
+        Args:
+            run: RunData object
+            timeout: number of seconds to wait
+
+        Returns:
+            RunData object
+        """
+
+        req = self._make_record(run=run)
         resp = self._request_response(req, timeout=timeout)
         return resp
 
+    def send_run_sync(self, run_obj, timeout=None):
+        run = self._make_run(run_obj)
+        return self._send_run_sync(run, timeout=timeout)
+
     def send_stats(self, stats_dict):
         stats = self._make_stats(stats_dict)
         rec = self._make_record(stats=stats)
@@ -303,9 +343,12 @@ class BackendSender(object):
     def send_exit(self, exit_code):
         pass
 
-    def send_exit_sync(self, exit_code, timeout=None):
-        exit = self._make_exit(exit_code)
-        req = self._make_record(exit=exit)
+    def _send_exit_sync(self, exit_data, timeout=None):
+        req = self._make_record(exit=exit_data)
 
         resp = self._request_response(req, timeout=timeout)
         return resp
+
+    def send_exit_sync(self, exit_code, timeout=None):
+        exit_data = self._make_exit(exit_code)
+        return self._send_exit_sync(exit_data, timeout=timeout)
