@@ -11,15 +11,20 @@ import logging
 import os
 import time
 
+from wandb.filesync.dir_watcher import DirWatcher
+from wandb.interface.interface import file_enum_to_policy
+from wandb.lib.config import save_config_file_from_dict
 from wandb.proto import wandb_internal_pb2  # type: ignore
 from wandb.util import sentry_set_scope
 
+
 # from wandb.stuff import io_wrap
 
 from . import artifacts
 from . import file_stream
 from . import internal_api
 from .file_pusher import FilePusher
+from .git_repo import GitRepo
 
 
 logger = logging.getLogger(__name__)
@@ -40,13 +45,14 @@ def _config_dict_from_proto_list(obj_list):
 
 
 class SendManager(object):
-    def __init__(self, settings, resp_q, run_meta):
+    def __init__(self, settings, resp_q, run_meta=None):
         self._settings = settings
         self._resp_q = resp_q
         self._run_meta = run_meta
 
         self._fs = None
         self._pusher = None
+        self._dir_watcher = None
 
         # is anyone using run_id?
         self._run_id = None
@@ -110,6 +116,10 @@ class SendManager(object):
         config_dict = None
         if run.HasField("config"):
             config_dict = _config_dict_from_proto_list(run.config.update)
+            config_path = os.path.join(self._settings.files_dir, "config.yaml")
+            save_config_file_from_dict(config_path, config_dict)
+
+        repo = GitRepo(remote=self._settings.git_remote)
 
         ups = self._api.upsert_run(
             name=run.run_id,
@@ -122,7 +132,10 @@ class SendManager(object):
             tags=run_tags or None,
             config=config_dict or None,
             sweep_name=run.sweep_id or None,
-            host=self._settings.host or None,
+            host=run.host or None,
+            program_path=self._settings.program or None,
+            repo=repo.remote_url,
+            commit=repo.last_commit,
         )
 
         if data.control.req_resp:
@@ -155,6 +168,7 @@ class SendManager(object):
         )
         self._fs.start()
         self._pusher = FilePusher(self._api)
+        self._dir_watcher = DirWatcher(self._settings, self._api, self._pusher)
         self._run_id = run.run_id
         if self._run_meta:
             self._run_meta.write()
@@ -172,8 +186,12 @@ class SendManager(object):
     def handle_summary(self, data):
         summary = data.summary
         summary_dict = _dict_from_proto_list(summary.update)
+        json_summary = json.dumps(summary_dict)
         if self._fs:
-            self._fs.push("wandb-summary.json", json.dumps(summary_dict))
+            self._fs.push("wandb-summary.json", json_summary)
+        summary_path = os.path.join(self._settings.files_dir, "wandb-summary.json")
+        with open(summary_path, "w") as f:
+            f.write(json_summary)
 
     def handle_stats(self, data):
         stats = data.stats
@@ -225,22 +243,22 @@ class SendManager(object):
         self._api.upsert_run(
             name=self._run_id, config=config_dict, **self._api_settings
         )
+        config_path = os.path.join(self._settings.files_dir, "config.yaml")
+        save_config_file_from_dict(config_path, config_dict)
         # TODO(jhr): check result of upsert_run?
 
-    def _save_file(self, fname):
+    def _save_file(self, fname, policy="end"):
         directory = self._settings.files_dir
         logger.info("saving file %s at %s", fname, directory)
         path = os.path.abspath(os.path.join(directory, fname))
         logger.info("saving file %s at full %s", fname, path)
-        self._pusher.file_changed(fname, path)
+        self._dir_watcher.update_policy(fname, policy)
 
     def handle_files(self, data):
         files = data.files
         for k in files.files:
-            fpath = k.path
             # TODO(jhr): fix paths with directories
-            fname = fpath[0]
-            self._save_file(fname)
+            self._save_file(k.path, file_enum_to_policy(k.policy))
 
     def handle_artifact(self, data):
         artifact = data.artifact
@@ -262,6 +280,8 @@ class SendManager(object):
         )
 
     def finish(self):
+        if self._dir_watcher:
+            self._dir_watcher.finish()
         if self._pusher:
             self._pusher.finish()
         if self._fs:
