@@ -27,12 +27,13 @@ import os
 import platform
 import socket
 import sys
+import tempfile
 
-import shortuuid  # type: ignore
 import six
 import wandb
-from wandb import jupyter
 from wandb.internal import git_repo
+from wandb.lib.ipython import _get_python_type
+from wandb.lib.runid import generate_id
 
 if wandb.TYPE_CHECKING:  # type: ignore
     from typing import (  # noqa: F401 pylint: disable=unused-import
@@ -59,12 +60,6 @@ source = (
 Field = collections.namedtuple("TypedField", ["type", "choices"])
 
 
-def _generate_id():
-    # ~3t run ids (36**8)
-    run_gen = shortuuid.ShortUUID(alphabet=list("0123456789abcdefghijklmnopqrstuvwxyz"))
-    return run_gen.random(8)
-
-
 defaults = dict(
     base_url="https://api.wandb.ai",
     show_warnings=2,
@@ -74,6 +69,10 @@ defaults = dict(
     console="auto",
     _console=Field(str, ("auto", "redirect", "off", "file", "iowrap",)),
     git_remote="origin",
+    ignore_globs=[],
+    # anonymous might be set by a config file: "false" and "true"
+    #   or from wandb.init(anonymous=) or environment: "allow", "must", "never"
+    _anonymous=Field(str, ("allow", "must", "never", "false", "true",)),
 )
 
 # env mapping?
@@ -95,13 +94,15 @@ env_settings = dict(
     host=None,
     username=None,
     disable_code=None,
-    wandb_dir="WANDB_DIR",
+    anonymous=None,
+    ignore_globs=None,
+    root_dir="WANDB_DIR",
     run_name="WANDB_NAME",
     run_notes="WANDB_NOTES",
     run_tags="WANDB_TAGS",
 )
 
-env_convert = dict(run_tags=lambda s: s.split(","),)
+env_convert = dict(run_tags=lambda s: s.split(","), ignore_globs=lambda s: s.split(","))
 
 
 def _build_inverse_map(prefix, d):
@@ -112,16 +113,6 @@ def _build_inverse_map(prefix, d):
     return inv_map
 
 
-def _get_python_type():
-    try:
-        if "terminal" in get_ipython().__module__:
-            return "ipython"
-        else:
-            return "jupyter"
-    except (NameError, AttributeError):
-        return "python"
-
-
 def _is_kaggle():
     return (
         os.getenv("KAGGLE_KERNEL_RUN_TYPE") is not None
@@ -137,12 +128,12 @@ def _get_program():
     try:
         import __main__  # type: ignore
 
-        program = __main__.__file__
-        if not program:
-            return "<python with no main file>"
+        return __main__.__file__
     except (ImportError, AttributeError):
         return None
 
+
+def _get_program_relpath_from_gitrepo(program):
     repo = git_repo.GitRepo()
     root = repo.root
     if not root:
@@ -161,6 +152,24 @@ def _get_program():
     return None
 
 
+# the setting exposed to users as `dir=` or `WANDB_DIR` is actually
+# the `root_dir`. We add the `__stage_dir__` to it to get the full
+# `wandb_dir`
+def get_wandb_dir(root_dir: str):
+    # We use the hidden version if it already exists, otherwise non-hidden.
+    if os.path.exists(os.path.join(root_dir, ".wandb")):
+        __stage_dir__ = ".wandb" + os.sep
+    else:
+        __stage_dir__ = "wandb" + os.sep
+
+    path = os.path.join(root_dir, __stage_dir__)
+    if not os.access(root_dir, os.W_OK):
+        wandb.termwarn("Path %s wasn't writable, using system temp directory" % path)
+        path = os.path.join(tempfile.gettempdir(), __stage_dir__ or ("wandb" + os.sep))
+
+    return path
+
+
 class CantTouchThis(type):
     def __setattr__(cls, attr, value):
         raise Exception("NO!")
@@ -208,7 +217,8 @@ class Settings(six.with_metaclass(CantTouchThis, object)):
         config_paths=None,
         _config_dict=None,
         # directories and files
-        wandb_dir="wandb",
+        root_dir=None,
+        wandb_dir=None,  # computed
         settings_system_spec="~/.config/wandb/settings",
         settings_workspace_spec="{wandb_dir}/settings",
         settings_system=None,  # computed
@@ -239,16 +249,20 @@ class Settings(six.with_metaclass(CantTouchThis, object)):
         program=None,
         notebook_name=None,
         disable_code=None,
+        ignore_globs=None,
         save_code=None,
-        code_program=None,
+        program_relpath=None,
         git_remote=None,
         host=None,
         username=None,
         docker=None,
         _start_time=None,
         _start_datetime=None,
+        _cli_only_mode=None,  # avoid running any code specific for runs
         console=None,
         disabled=None,  # alias for mode=dryrun, not supported yet
+        reinit=None,
+        _save_requirements=True,
         # compute environment
         jupyter=None,
         windows=None,
@@ -299,7 +313,7 @@ class Settings(six.with_metaclass(CantTouchThis, object)):
             )
             self.update(self._load(settings_workspace), _setter="workspace")
         if _settings:
-            self.update(_settings)
+            self.update(dict(_settings))
         if _environ:
             _logger = _early_logger or logger
             inv_map = _build_inverse_map(env_prefix, env_settings)
@@ -338,7 +352,9 @@ class Settings(six.with_metaclass(CantTouchThis, object)):
         if self.run_id:
             format_dict["run_id"] = self.run_id
         format_dict["proc"] = os.getpid()
-        format_dict["wandb_dir"] = self.wandb_dir
+        # TODO(cling): hack to make sure we read from local settings
+        #              this is wrong if the run_dir changes later
+        format_dict["wandb_dir"] = self.wandb_dir or "wandb"
 
         path_items = []
         for p in path:
@@ -388,7 +404,8 @@ class Settings(six.with_metaclass(CantTouchThis, object)):
         self.__dict__.update({k: v for k, v in d.items() if v is not None})
         self.__dict__.update({k: v for k, v in kwargs.items() if v is not None})
 
-    def infer_settings_from_env(self):
+    def _infer_settings_from_env(self):
+        """Modify settings based on environment (for runs and cli)."""
         d = {}
         d["jupyter"] = _get_python_type() != "python"
         d["windows"] = platform.system() == "Windows"
@@ -414,7 +431,7 @@ class Settings(six.with_metaclass(CantTouchThis, object)):
             u["save_code"] = wandb.env.should_save_code()
 
         if self.jupyter:
-            meta = jupyter.notebook_metadata()
+            meta = wandb.jupyter.notebook_metadata()
             u["_jupyter_path"] = meta.get("path")
             u["_jupyter_name"] = meta.get("name")
             u["_jupyter_root"] = meta.get("root")
@@ -434,7 +451,7 @@ class Settings(six.with_metaclass(CantTouchThis, object)):
 
         u["_executable"] = sys.executable
 
-        u["docker"] = wandb.env.get_docker()
+        u["docker"] = wandb.env.get_docker(wandb.util.image_id_from_k8s())
 
         # TODO: we should use the cuda library to collect this
         if os.path.exists("/usr/local/cuda/version.txt"):
@@ -446,11 +463,18 @@ class Settings(six.with_metaclass(CantTouchThis, object)):
 
         self.update(u)
 
+    def _infer_run_settings_from_env(self):
+        """Modify settings based on environment (for runs only)."""
         # If the settings say to save code, and there's not already a program file,
         # infer it now.
-        if self.save_code and not self.code_program:
-            code_program = _get_program()
-            self.update(dict(code_program=code_program))
+        if self.save_code and not self.program_relpath:
+            program = _get_program()
+            if program:
+                program_relpath = _get_program_relpath_from_gitrepo(program)
+                self.update(dict(program=program, program_relpath=program_relpath))
+            else:
+                program = "<python with no main file>"
+                self.update(dict(program=program))
 
     def setdefaults(self, __d=None):
         __d = __d or defaults
@@ -496,6 +520,9 @@ class Settings(six.with_metaclass(CantTouchThis, object)):
         d = dict()
         for k in cp[section]:
             d[k] = cp[section][k]
+            # TODO (cvp): we didn't do this in the old cli, but it seems necessary
+            if k == "ignore_globs":
+                d[k] = d[k].split(",")
         return d
 
     def apply_init(self, args):
@@ -505,8 +532,9 @@ class Settings(six.with_metaclass(CantTouchThis, object)):
             id="run_id",
             tags="run_tags",
             group="run_group",
-            dir="wandb_dir",
+            dir="root_dir",
         )
         args = {param_map.get(k, k): v for k, v in six.iteritems(args) if v is not None}
         self.update(args)
-        self.run_id = self.run_id or _generate_id()
+        self.run_id = self.run_id or generate_id()
+        self.wandb_dir = get_wandb_dir(self.root_dir or ".")
