@@ -44,6 +44,7 @@ import wandb
 import wandb.old.core
 from wandb.old.core import wandb_dir
 from wandb.errors.error import CommError
+from wandb.internal.git_repo import GitRepo
 # from wandb import wandb_config
 from wandb import env
 
@@ -288,6 +289,29 @@ def ensure_matplotlib_figure(obj):
     """
     import matplotlib
     from matplotlib.figure import Figure
+    # plotly and matplotlib broke in recent releases,
+    # this patches matplotlib to add a removed method that plotly assumes exists
+    from matplotlib.spines import Spine
+    def is_frame_like(self):
+        """Return True if directly on axes frame.
+
+        This is useful for determining if a spine is the edge of an
+        old style MPL plot. If so, this function will return True.
+        """
+        position = self._position or ('outward', 0.0)
+        if isinstance(position, str):
+            if position == 'center':
+                position = ('axes', 0.5)
+            elif position == 'zero':
+                position = ('data', 0)
+        if len(position) != 2:
+            raise ValueError("position should be 2-tuple")
+        position_type, amount = position
+        if position_type == 'outward' and amount == 0:
+            return True
+        else:
+            return False
+    Spine.is_frame_like = is_frame_like
     if obj == matplotlib.pyplot:
         obj = obj.gcf()
     elif not isinstance(obj, Figure):
@@ -310,7 +334,10 @@ def json_friendly(obj):
     if is_tf_eager_tensor_typename(typename):
         obj = obj.numpy()
     elif is_tf_tensor_typename(typename):
-        obj = obj.eval()
+        try:
+            obj = obj.eval()
+        except RuntimeError:
+            obj = obj.numpy()
     elif is_pytorch_tensor_typename(typename):
         try:
             if obj.requires_grad:
@@ -572,48 +599,6 @@ def no_retry_auth(e):
         raise CommError("Permission denied, ask the project owner to grant you access")
 
 
-def write_netrc(host, entity, key):
-    """Add our host and key to .netrc"""
-    key_prefix, key_suffix = key.split('-', 1) if '-' in key else ('', key)
-    if len(key_suffix) != 40:
-        wandb.termlog('API-key must be exactly 40 characters long: {} ({} chars)'.format(key_suffix, len(key_suffix)))
-        return None
-    try:
-        normalized_host = host.split("/")[-1].split(":")[0]
-        wandb.termlog("Appending key for {} to your netrc file: {}".format(
-            normalized_host, os.path.expanduser('~/.netrc')))
-        machine_line = 'machine %s' % normalized_host
-        path = os.path.expanduser('~/.netrc')
-        orig_lines = None
-        try:
-            with open(path) as f:
-                orig_lines = f.read().strip().split('\n')
-        except (IOError, OSError) as e:
-            pass
-        with open(path, 'w') as f:
-            if orig_lines:
-                # delete this machine from the file if it's already there.
-                skip = 0
-                for line in orig_lines:
-                    if machine_line in line:
-                        skip = 2
-                    elif skip:
-                        skip -= 1
-                    else:
-                        f.write('%s\n' % line)
-            f.write(textwrap.dedent("""\
-            machine {host}
-              login {entity}
-              password {key}
-            """).format(host=normalized_host, entity=entity, key=key))
-        os.chmod(os.path.expanduser('~/.netrc'),
-                 stat.S_IRUSR | stat.S_IWUSR)
-        return True
-    except IOError as e:
-        wandb.termerror("Unable to read ~/.netrc")
-        return None
-
-
 def request_with_retry(func, *args, **kwargs):
     """Perform a requests http call, retrying with exponential backoff.
 
@@ -892,112 +877,17 @@ def download_file_from_url(dest_path, source_url, api_key=None):
     response = requests.get(source_url, auth=("api", api_key), stream=True, timeout=5)
     response.raise_for_status()
 
-    if "/" in dest_path:
-        dir = "/".join(dest_path.split("/")[0:-1])
-        mkdir_exists_ok(dir)
+    if os.sep in dest_path:
+        mkdir_exists_ok(os.path.dirname(dest_path))
     with open(dest_path, "wb") as file:
         for data in response.iter_content(chunk_size=1024):
             file.write(data)
 
 
-def set_api_key(api, key, anonymous=False):
-    if not key:
-        return
-
-    # Normal API keys are 40-character hex strings. Onprem API keys have a
-    # variable-length prefix, a dash, then the 40-char string.
-    prefix, suffix = key.split('-') if '-' in key else ('', key)
-
-    if len(suffix) == 40:
-        os.environ[env.API_KEY] = key
-        api.set_setting('anonymous', str(anonymous).lower(), globally=True)
-        write_netrc(api.api_url, "user", key)
-        api.reauth()
-        return
-    raise ValueError("API key must be 40 characters long, yours was %s" % len(key))
-
-
 def isatty(ob):
     return hasattr(ob, "isatty") and ob.isatty()
 
 
-LOGIN_CHOICE_ANON = 'Private W&B dashboard, no account required'
-LOGIN_CHOICE_NEW = 'Create a W&B account'
-LOGIN_CHOICE_EXISTS = 'Use an existing W&B account'
-LOGIN_CHOICE_DRYRUN = "Don't visualize my results"
-LOGIN_CHOICES = [
-    LOGIN_CHOICE_ANON,
-    LOGIN_CHOICE_NEW,
-    LOGIN_CHOICE_EXISTS,
-    LOGIN_CHOICE_DRYRUN
-]
-
-
-def prompt_api_key(api, input_callback=None, browser_callback=None, no_offline=False, local=False):
-    input_callback = input_callback or getpass.getpass
-
-    choices = [choice for choice in LOGIN_CHOICES]
-    if os.environ.get(env.ANONYMOUS, "never") == "never":
-        # Omit LOGIN_CHOICE_ANON as a choice if the env var is set to never
-        choices.remove(LOGIN_CHOICE_ANON)
-    if os.environ.get(env.JUPYTER, "false") == "true" or no_offline:
-        choices.remove(LOGIN_CHOICE_DRYRUN)
-
-    if os.environ.get(env.ANONYMOUS) == "must":
-        result = LOGIN_CHOICE_ANON
-    # If we're not in an interactive environment, default to dry-run.
-    elif not isatty(sys.stdout) or not isatty(sys.stdin):
-        result = LOGIN_CHOICE_DRYRUN
-    elif local:
-        result = LOGIN_CHOICE_EXISTS
-    else:
-        for i, choice in enumerate(choices):
-            wandb.termlog("(%i) %s" % (i + 1, choice))
-
-        def prompt_choice():
-            try:
-                return int(six.moves.input("%s: Enter your choice: " % wandb.core.LOG_STRING)) - 1
-            except ValueError:
-                return -1
-        idx = -1
-        while idx < 0 or idx > len(choices) - 1:
-            idx = prompt_choice()
-            if idx < 0 or idx > len(choices) - 1:
-                wandb.termwarn("Invalid choice")
-        result = choices[idx]
-        wandb.termlog("You chose '%s'" % result)
-
-    if result == LOGIN_CHOICE_ANON:
-        key = api.create_anonymous_api_key()
-
-        set_api_key(api, key, anonymous=True)
-        return key
-    elif result == LOGIN_CHOICE_NEW:
-        key = browser_callback(signup=True) if browser_callback else None
-
-        if not key:
-            wandb.termlog('Create an account here: {}/authorize?signup=true'.format(api.app_url))
-            key = input_callback('%s: Paste an API key from your profile and hit enter' % wandb.core.LOG_STRING).strip()
-
-        set_api_key(api, key)
-        return key
-    elif result == LOGIN_CHOICE_EXISTS:
-        key = browser_callback() if browser_callback else None
-
-        if not key:
-            wandb.termlog('You can find your API key in your browser here: {}/authorize'.format(api.app_url))
-            key = input_callback('%s: Paste an API key from your profile and hit enter' % wandb.core.LOG_STRING).strip()
-        set_api_key(api, key)
-        return key
-    else:
-        # Jupyter environments don't have a tty, but we can still try logging in using the browser callback if one
-        # is supplied.
-        key, anonymous = browser_callback() if os.environ.get(env.JUPYTER, "false") == "true" and browser_callback else (None, False)
-
-        set_api_key(api, key, anonymous=anonymous)
-        return key
-
-
 def sizeof_fmt(num, suffix='B'):
     """Pretty print file size
         https://stackoverflow.com/questions/1094841/reusable-library-to-get-human-readable-version-of-file-size
@@ -1009,9 +899,10 @@ def sizeof_fmt(num, suffix='B'):
     return "%.1f%s%s" % (num, 'Yi', suffix)
 
 
-def auto_project_name(program, api):
+def auto_project_name(program):
     # if we're in git, set project name to git repo name + relative path within repo
-    root_dir = api.git.root_dir
+    repo = GitRepo()
+    root_dir = repo.root_dir
     if root_dir is None:
         return None
     repo_name = os.path.basename(root_dir)
@@ -1066,4 +957,4 @@ def to_forward_slash_path(path):
 
 def bytes_to_hex(bytestr):
     # Works in python2 / python3
-    return codecs.getencoder('hex')(bytestr)[0].decode('ascii')
\ No newline at end of file
+    return codecs.getencoder('hex')(bytestr)[0].decode('ascii')
